@@ -167,7 +167,7 @@ class _RetryWrapper(object):
             'Tasklet has exceeded request deadline after %s seconds total',
             time.time() - start_time)
         raise
-      except self.retriable_exceptions as e:
+      except self.retriable_exceptions, e:
         pass
 
       if n == 1:
